'use strict';

var parser = require('@babel/parser');
var micromatch = require('micromatch');
var path = require('node:path');
var twin_setup = require('../jsx/twin/twin.setup');

function _interopDefaultCompat (e) { return e && typeof e === 'object' && 'default' in e ? e : { default: e }; }

var micromatch__default = /*#__PURE__*/_interopDefaultCompat(micromatch);
var path__default = /*#__PURE__*/_interopDefaultCompat(path);

let twin = undefined;
const allowedFileRegex = /^(?!.*[/\\](react|react-native|react-native-web|@native-twin\/*)[/\\]).*$/;
const isValidFile = (x = '', allowedFiles) => {
    if (!micromatch__default.default.isMatch(x, allowedFiles)) {
        return false;
    }
    return allowedFileRegex.test(x);
};
const createVisitorStateContext = (state, allowedPaths) => {
    return {
        isValidPath: isValidFile(state.filename, allowedPaths),
    };
};
const createVisitorContext = (rootPath, options) => {
    const twConfig = twin_setup.getUserTwinConfig(rootPath, options);
    twin = twin_setup.setupNativeTwin(twConfig, options);
    const allowedPaths = twConfig.content.map((x) => path__default.default.resolve(rootPath, path__default.default.join(x)));
    return (path, state) => {
        const stateContext = createVisitorStateContext(state, allowedPaths);
        return {
            stateContext,
            twConfig,
            twin,
            tree: {
                root: path,
                parent: path.parent,
            },
            options: options,
        };
    };
};
const createBabelAST = (code) => {
    return parser.parse(code, {
        plugins: ['jsx', 'typescript'],
        sourceType: 'module',
        errorRecovery: true,
        tokens: false,
    });
};

exports.createBabelAST = createBabelAST;
exports.createVisitorContext = createVisitorContext;
//# sourceMappingURL=babel.common.js.map
