import { parse } from '@babel/parser';
import micromatch from 'micromatch';
import path from 'node:path';
import { getUserTwinConfig, setupNativeTwin } from '../jsx/twin/twin.setup';

let twin = undefined;
const allowedFileRegex = /^(?!.*[/\\](react|react-native|react-native-web|@native-twin\/*)[/\\]).*$/;
const isValidFile = (x = '', allowedFiles) => {
    if (!micromatch.isMatch(x, allowedFiles)) {
        return false;
    }
    return allowedFileRegex.test(x);
};
const createVisitorStateContext = (state, allowedPaths) => {
    return {
        isValidPath: isValidFile(state.filename, allowedPaths),
    };
};
const createVisitorContext = (rootPath, options) => {
    const twConfig = getUserTwinConfig(rootPath, options);
    twin = setupNativeTwin(twConfig, options);
    const allowedPaths = twConfig.content.map((x) => path.resolve(rootPath, path.join(x)));
    return (path, state) => {
        const stateContext = createVisitorStateContext(state, allowedPaths);
        return {
            stateContext,
            twConfig,
            twin,
            tree: {
                root: path,
                parent: path.parent,
            },
            options: options,
        };
    };
};
const createBabelAST = (code) => {
    return parse(code, {
        plugins: ['jsx', 'typescript'],
        sourceType: 'module',
        errorRecovery: true,
        tokens: false,
    });
};

export { createBabelAST, createVisitorContext };
//# sourceMappingURL=babel.common.js.map
