import * as t from '@babel/types';
import * as Option from 'effect/Option';
import { createPrimitiveExpression } from './babel.constructors';

const isReactNativeImport = (binding) => {
    return (t.isImportSpecifier(binding.path.node) &&
        t.isImportDeclaration(binding.path.parent) &&
        binding.path.parent.source.value === 'react-native');
};
const isReactNativeRequire = (binding) => {
    return (t.isCallExpression(binding.path.node) &&
        t.isIdentifier(binding.path.node.callee, { name: 'require' })
    // binding.path.parent.source.value === 'react-native'
    );
};
const isReactRequire = (node) => {
    return node.pipe(Option.filterMap(([, node]) => t.isIdentifier(node.callee, { name: 'require' }) &&
        t.isStringLiteral(node.arguments[0], { value: 'react' })
        ? Option.some(true)
        : Option.none()));
};
const isReactInteropRequire = (node) => {
    return node.pipe(Option.flatMap(([, node]) => {
        if (
        // const <name> = _interopRequireDefault(require("react"))
        t.isIdentifier(node.callee, { name: '_interopRequireDefault' }) &&
            t.isCallExpression(node.arguments[0]) &&
            t.isIdentifier(node.arguments[0].callee, { name: 'require' }) &&
            t.isStringLiteral(node.arguments[0].arguments[0], {
                value: 'react',
            })) {
            return Option.some(true);
        }
        return Option.none();
    }));
};
const valueIsPrimitive = (value) => {
    return (typeof value === 'string' || typeof value === 'number' || typeof value === 'boolean');
};
const hasJsxAttribute = (element, name, value) => {
    if (!t.isJSXElement(element))
        return false;
    return element.openingElement.attributes.some((x) => t.isJSXAttribute(x, {
        name: t.jsxIdentifier(name),
        value: t.jsxExpressionContainer(createPrimitiveExpression(value)),
    }));
};

export { hasJsxAttribute, isReactInteropRequire, isReactNativeImport, isReactNativeRequire, isReactRequire, valueIsPrimitive };
//# sourceMappingURL=babel.predicates.js.map
