import * as Context from 'effect/Context';
import * as Effect from 'effect/Effect';
import { pipe } from 'effect/Function';
import * as HashMap from 'effect/HashMap';
import * as Layer from 'effect/Layer';
import * as Option from 'effect/Option';
import micromatch from 'micromatch';
import path from 'node:path';
import { maybeCreateElementExpression, maybeReactIdent, maybeBinding, maybeImportDeclaration } from '../babel';
import { getUserTwinConfig, setupNativeTwin } from '../jsx/twin';
import { maybeBindingIsReactImport, isReactRequireBinding, isReactImport } from './import.utils';

class JSXImportPluginContext extends Context.Tag('babel/plugin/context')() {
    static make = (options, rootPath) => Layer.scoped(JSXImportPluginContext, Effect.gen(function* () {
        const twConfig = getUserTwinConfig(rootPath, options);
        const twin = setupNativeTwin(twConfig, options);
        const allowedPaths = twConfig.content.map((x) => path.resolve(rootPath, path.join(x)));
        const twCtx = {
            baseRem: twin.config.root.rem,
            platform: options.platform,
        };
        const visitedElements = HashMap.empty();
        return {
            options,
            rootPath,
            twin,
            twConfig: twin.config,
            allowedPaths,
            twCtx,
            visitedElements,
            isValidFile(filename = '') {
                const allowedFileRegex = /^(?!.*[/\\](react|react-native|react-native-web|@native-twin\/*)[/\\]).*$/;
                if (!micromatch.isMatch(filename, allowedPaths)) {
                    return false;
                }
                return allowedFileRegex.test(filename);
            },
        };
    }));
}
class BabelJSXImportPlugin extends Context.Tag('babel/plugin/import-service')() {
    static Live = Layer.scoped(BabelJSXImportPlugin, Effect.gen(function* () {
        return {
            memberExpressionIsReactImport: (path) => {
                return maybeCreateElementExpression(path)
                    .pipe(maybeReactIdent, (x) => maybeBinding(x, path))
                    .pipe((x) => [maybeBindingIsReactImport(x), maybeImportDeclaration(x)], Option.firstSomeOf, Option.getOrElse(() => false));
            },
            identifierIsReactImport: (path) => {
                if (path.node.name === 'createElement' && path.parentPath.isCallExpression()) {
                    return pipe(Option.fromNullable(path.scope.getBinding(path.node.name)), Option.map((x) => isReactRequireBinding(x) || isReactImport(x)), Option.getOrElse(() => false));
                }
                return false;
            },
        };
    }));
}

export { BabelJSXImportPlugin, JSXImportPluginContext };
//# sourceMappingURL=JSXImport.service.js.map
