import * as t from '@babel/types';
import { getRawSheet } from '@native-twin/css/jsx';
import { createPrimitiveExpression, hasJsxAttribute } from '../../babel';
import { entriesToComponentData, runtimeEntriesToAst } from '../twin';

const createRequireExpression = (path) => {
    return t.callExpression(t.identifier('require'), [t.stringLiteral(path)]);
};
const createJsxAttribute = (name, value) => {
    const expression = createPrimitiveExpression(value);
    return t.jsxAttribute(t.jsxIdentifier(name), t.jsxExpressionContainer(expression));
};
const addJsxAttribute = (element, name, value) => {
    if (!t.isJSXElement(element))
        return;
    const newAttribute = createJsxAttribute(name, value);
    if (hasJsxAttribute(element, name, value)) {
        element.openingElement.attributes = element.openingElement.attributes.map((x) => {
            if (x.type === 'JSXSpreadAttribute')
                return x;
            if (x.type === 'JSXAttribute' &&
                x.name.type === 'JSXIdentifier' &&
                x.name.name === name) {
                return newAttribute;
            }
            return x;
        });
        return;
    }
    element.openingElement.attributes.push(newAttribute);
};
const addJsxExpressionAttribute = (element, name, value) => {
    if (!t.isJSXElement(element))
        return;
    const newAttribute = t.jsxAttribute(t.jsxIdentifier(name), t.jsxExpressionContainer(value));
    if (JSXElementHasAttribute(element, name)) {
        console.log('ALREADY_HAS: ', name);
        element.openingElement.attributes = element.openingElement.attributes.map((x) => {
            if (x.type === 'JSXSpreadAttribute')
                return x;
            if (x.type === 'JSXAttribute' &&
                x.name.type === 'JSXIdentifier' &&
                x.name.name === name) {
                return newAttribute;
            }
            return x;
        });
        return;
    }
    element.openingElement.attributes.push(newAttribute);
};
const JSXElementHasAttribute = (element, name) => {
    return element.openingElement.attributes.some((x) => x.type === 'JSXAttribute' &&
        x.name.type === 'JSXIdentifier' &&
        x.name.name === name);
};
function addTwinPropsToElement(elementNode, entries, options) {
    const stringEntries = entriesToComponentData(elementNode.id, getRawSheet(entries));
    const astProps = runtimeEntriesToAst(stringEntries.styledProp);
    // const treeProp = elementNodeToTree(elementNode, filename, elementNode.);
    if (options.componentID) {
        addJsxAttribute(elementNode.path, '_twinComponentID', elementNode.id);
    }
    if (options.order) {
        addJsxAttribute(elementNode.path, '_twinOrd', elementNode.order);
    }
    if (options.styledProps && astProps) {
        addJsxExpressionAttribute(elementNode.path, '_twinComponentSheet', astProps);
    }
    if (options.templateStyles && stringEntries.templateEntries) {
        const astTemplate = runtimeEntriesToAst(stringEntries.templateEntries);
        if (astTemplate) {
            addJsxExpressionAttribute(elementNode.path, '_twinComponentTemplateEntries', astTemplate);
        }
    }
    return astProps;
}
function compileCssForElement(elementNode, entries, options) {
    // const stringEntries = entriesToObject(elementNode.id, getRawSheet(entries));
    // const astProps = runtimeEntriesToAst(stringEntries.styledProp);
    // const treeProp = elementNodeToTree(elementNode, filename, elementNode.);
    // const astTemplate = runtimeEntriesToAst(stringEntries.templateEntries);
    //
    // if (options.componentID) {
    //   addJsxAttribute(elementNode.path, '_twinComponentID', elementNode.id);
    // }
    // if (options.order) {
    //   addJsxAttribute(elementNode.path, '_twinOrd', elementNode.order);
    // }
}

export { JSXElementHasAttribute, addJsxAttribute, addJsxExpressionAttribute, addTwinPropsToElement, compileCssForElement, createJsxAttribute, createRequireExpression };
//# sourceMappingURL=jsx.builder.js.map
