import * as Option from 'effect/Option';
import fs from 'node:fs';
import path from 'node:path';
import * as NativeTwin from '@native-twin/core';
import { TWIN_DEFAULT_FILES, TWIN_INPUT_CSS_FILE } from '../../constants';
import { requireJS } from '../../utils/load-js';

let tw = null;
function loadNativeTwin() {
    return requireJS('@native-twin/core');
}
function loadVirtualSheet() {
    return requireJS('@native-twin/css').createVirtualSheet();
}
function loadNativeTwinConfig(mod, config, options) {
    return mod.createTailwind(config, loadVirtualSheet());
}
function getUserTwinConfig(rootDir, options) {
    const twinConfigPath = getTwinConfigPath(rootDir, options);
    return twinConfigPath.pipe(Option.map((x) => requireJS(x)), Option.map((x) => NativeTwin.defineConfig({
        ...x,
        mode: options.platform === 'web' ? 'web' : x.mode ?? 'native',
        root: { rem: x.root.rem ?? 16 },
    })), Option.getOrElse(() => NativeTwin.defineConfig({
        content: [],
        mode: options.platform === 'web' ? 'web' : 'native',
        root: { rem: 16 },
    })));
}
const getTwinConfigPath = (rootDir, options) => {
    let twPath;
    if (options.twinConfigPath) {
        twPath = path.join(rootDir, options.twinConfigPath);
    }
    if (!twPath) {
        twPath = TWIN_DEFAULT_FILES.map((x) => path.join(rootDir, x)).find((x) => fs.existsSync(x));
    }
    return Option.fromNullable(twPath).pipe(Option.map((x) => path.resolve(x)));
};
function setupNativeTwin(config, options) {
    if (tw) {
        return tw;
    }
    const nativeTwin = loadNativeTwin();
    tw = loadNativeTwinConfig(nativeTwin, config);
    return tw;
}
const createTwinCSSFiles = ({ outputDir, inputCSS, }) => {
    if (!fs.existsSync(path.resolve(outputDir))) {
        fs.mkdirSync(outputDir, { recursive: true });
    }
    if (!inputCSS) {
        inputCSS = path.join(outputDir, TWIN_INPUT_CSS_FILE);
        fs.writeFileSync(inputCSS, '');
    }
    // fs.writeFileSync(outputCSS, '.tt_root { font-size: 16px }', 'utf-8');
    return {
        inputCSS,
    };
};
// export const createCacheDir = (outputDir: string) => {
//   if (!fs.existsSync(path.resolve(outputDir))) {
//     fs.mkdirSync(outputDir, { recursive: true });
//   }
//   fs.writeFileSync(path.join(outputDir, TWIN_STYLES_FILE), twinModuleExportString);
// };
const deleteCacheDir = (outputDir) => {
    if (!fs.existsSync(path.resolve(outputDir))) {
        fs.rmdirSync(outputDir, { recursive: true });
    }
};
const getTwinCacheDir = () => path.join(path.dirname(require.resolve('@native-twin/core')), '.cache');

export { createTwinCSSFiles, deleteCacheDir, getTwinCacheDir, getUserTwinConfig, setupNativeTwin };
//# sourceMappingURL=twin.setup.js.map
