import * as Equal from 'effect/Equal';
import { pipe } from 'effect/Function';
import * as Hash from 'effect/Hash';
import * as Option from 'effect/Option';
import { getChildRuntimeEntries } from '@native-twin/css/jsx';
import { getJSXElementAttrs, getJSXElementName } from '../jsx/ast/jsx.maps';
import { getElementEntries } from '../jsx/twin';

const jsxElementHash = (path, filename) => {
    const filenameHash = Hash.string(`${filename}`);
    return pipe(Hash.string(`${path.start}-${path.end}`), Hash.combine(filenameHash), Hash.combine(Hash.string(path.type)));
};
class JSXElementNodeKey {
    path;
    filename;
    constructor(path, filename) {
        this.path = path;
        this.filename = filename;
    }
    [Hash.symbol]() {
        return jsxElementHash(this.path, this.filename);
    }
    [Equal.symbol](that) {
        return (that instanceof JSXElementNodeKey &&
            this.path.loc === that.path.loc &&
            this.path.type === that.path.type &&
            this.path.start == that.path.start);
    }
}
const jsxElementNodeKey = (path, filename) => new JSXElementNodeKey(path, filename);
class JSXElementNode {
    leave;
    order;
    filename;
    id;
    runtimeData;
    entries;
    childEntries;
    constructor({ filename, leave, order, runtimeData, twin }) {
        this.leave = leave;
        this.filename = filename;
        this.order = order;
        this.runtimeData = runtimeData;
        // this.id = `${jsxElementHash(leave.value.babelNode, filename)}`;
        this.id = leave.value.uid;
        this.entries = pipe(getElementEntries(this.runtimeData, twin.tw, twin.context));
        this.childEntries = getChildRuntimeEntries(this.entries);
    }
    get source() {
        return this.leave.value.source;
    }
    get parentSize() {
        return pipe(this.parentLeave, Option.map((x) => x.childrenCount), Option.getOrElse(() => 0));
    }
    get parentLeave() {
        return Option.fromNullable(this.leave.parent);
    }
    get parentID() {
        return Option.fromNullable(this.leave.value.parentID);
    }
    get path() {
        return this.leave.value.babelNode;
    }
    get attributes() {
        return getJSXElementAttrs(this.path);
    }
    get openingElement() {
        return this.path.openingElement;
    }
    get elementName() {
        return pipe(getJSXElementName(this.path.openingElement), Option.getOrElse(() => 'UnknownJSX'));
    }
    [Hash.symbol]() {
        return Hash.hash(this.id);
    }
    [Equal.symbol](that) {
        return that instanceof JSXElementNode && this.id === that.id;
    }
}

export { JSXElementNode, JSXElementNodeKey, jsxElementNodeKey };
//# sourceMappingURL=JSXElement.model.js.map
