'use strict';

var helperModuleImports = require('@babel/helper-module-imports');
var Effect = require('effect/Effect');
var babel = require('@native-twin/compiler/babel');
var plugin_constants = require('./constants/plugin.constants');
var JSXImport_service = require('./jsx-import/JSXImport.service');

function _interopNamespaceCompat(e) {
    if (e && typeof e === 'object' && 'default' in e) return e;
    var n = Object.create(null);
    if (e) {
        Object.keys(e).forEach(function (k) {
            if (k !== 'default') {
                var d = Object.getOwnPropertyDescriptor(e, k);
                Object.defineProperty(n, k, d.get ? d : {
                    enumerable: true,
                    get: function () { return e[k]; }
                });
            }
        });
    }
    n.default = e;
    return Object.freeze(n);
}

var Effect__namespace = /*#__PURE__*/_interopNamespaceCompat(Effect);

const program = Effect__namespace.scoped(Effect__namespace.gen(function* () {
    const ctx = yield* JSXImport_service.JSXImportPluginContext;
    const reactCompiler = yield* babel.ReactCompilerService;
    return {
        name: '@native-twin/babel-plugin',
        visitor: {
            MemberExpression(path, state) {
                if (!ctx.isValidFile(state.filename))
                    return;
                if (reactCompiler.memberExpressionIsReactImport(path)) {
                    path.replaceWith(helperModuleImports.addNamed(path, ...plugin_constants.PLUGIN_IMPORT_META));
                }
            },
            Identifier(path, state) {
                if (!ctx.isValidFile(state.filename))
                    return;
                if (reactCompiler.identifierIsReactImport(path)) {
                    path.replaceWith(helperModuleImports.addNamed(path, ...plugin_constants.PLUGIN_IMPORT_META));
                }
            },
        },
    };
}));
// const layer = Logger.replace(Logger.defaultLogger, BabelLogger);
function nativeTwinBabelPlugin(_, options, cwd) {
    return program.pipe(
    // Logger.withMinimumLogLevel(LogLevel.All),
    // Effect.provide(layer),
    Effect__namespace.provide(JSXImport_service.JSXImportPluginContext.make(options, cwd)), Effect__namespace.provide(babel.makeBabelLayer), Effect__namespace.runSync);
}

module.exports = nativeTwinBabelPlugin;
//# sourceMappingURL=index.js.map
