'use strict';

var Context = require('effect/Context');
var Effect = require('effect/Effect');
var Function = require('effect/Function');
var HashMap = require('effect/HashMap');
var Layer = require('effect/Layer');
var Option = require('effect/Option');
var micromatch = require('micromatch');
var path = require('node:path');
var babel = require('../babel');
var twin = require('../jsx/twin');
var import_utils = require('./import.utils');

function _interopDefaultCompat (e) { return e && typeof e === 'object' && 'default' in e ? e : { default: e }; }

function _interopNamespaceCompat(e) {
    if (e && typeof e === 'object' && 'default' in e) return e;
    var n = Object.create(null);
    if (e) {
        Object.keys(e).forEach(function (k) {
            if (k !== 'default') {
                var d = Object.getOwnPropertyDescriptor(e, k);
                Object.defineProperty(n, k, d.get ? d : {
                    enumerable: true,
                    get: function () { return e[k]; }
                });
            }
        });
    }
    n.default = e;
    return Object.freeze(n);
}

var Context__namespace = /*#__PURE__*/_interopNamespaceCompat(Context);
var Effect__namespace = /*#__PURE__*/_interopNamespaceCompat(Effect);
var HashMap__namespace = /*#__PURE__*/_interopNamespaceCompat(HashMap);
var Layer__namespace = /*#__PURE__*/_interopNamespaceCompat(Layer);
var Option__namespace = /*#__PURE__*/_interopNamespaceCompat(Option);
var micromatch__default = /*#__PURE__*/_interopDefaultCompat(micromatch);
var path__default = /*#__PURE__*/_interopDefaultCompat(path);

class JSXImportPluginContext extends Context__namespace.Tag('babel/plugin/context')() {
    static make = (options, rootPath) => Layer__namespace.scoped(JSXImportPluginContext, Effect__namespace.gen(function* () {
        const twConfig = twin.getUserTwinConfig(rootPath, options);
        const twin$1 = twin.setupNativeTwin(twConfig, options);
        const allowedPaths = twConfig.content.map((x) => path__default.default.resolve(rootPath, path__default.default.join(x)));
        const twCtx = {
            baseRem: twin$1.config.root.rem,
            platform: options.platform,
        };
        const visitedElements = HashMap__namespace.empty();
        return {
            options,
            rootPath,
            twin: twin$1,
            twConfig: twin$1.config,
            allowedPaths,
            twCtx,
            visitedElements,
            isValidFile(filename = '') {
                const allowedFileRegex = /^(?!.*[/\\](react|react-native|react-native-web|@native-twin\/*)[/\\]).*$/;
                if (!micromatch__default.default.isMatch(filename, allowedPaths)) {
                    return false;
                }
                return allowedFileRegex.test(filename);
            },
        };
    }));
}
class BabelJSXImportPlugin extends Context__namespace.Tag('babel/plugin/import-service')() {
    static Live = Layer__namespace.scoped(BabelJSXImportPlugin, Effect__namespace.gen(function* () {
        return {
            memberExpressionIsReactImport: (path) => {
                return babel.maybeCreateElementExpression(path)
                    .pipe(babel.maybeReactIdent, (x) => babel.maybeBinding(x, path))
                    .pipe((x) => [import_utils.maybeBindingIsReactImport(x), babel.maybeImportDeclaration(x)], Option__namespace.firstSomeOf, Option__namespace.getOrElse(() => false));
            },
            identifierIsReactImport: (path) => {
                if (path.node.name === 'createElement' && path.parentPath.isCallExpression()) {
                    return Function.pipe(Option__namespace.fromNullable(path.scope.getBinding(path.node.name)), Option__namespace.map((x) => import_utils.isReactRequireBinding(x) || import_utils.isReactImport(x)), Option__namespace.getOrElse(() => false));
                }
                return false;
            },
        };
    }));
}

exports.BabelJSXImportPlugin = BabelJSXImportPlugin;
exports.JSXImportPluginContext = JSXImportPluginContext;
//# sourceMappingURL=JSXImport.service.js.map
