'use strict';

var RA = require('effect/Array');
var Function = require('effect/Function');
var Predicate = require('effect/Predicate');

function _interopNamespaceCompat(e) {
    if (e && typeof e === 'object' && 'default' in e) return e;
    var n = Object.create(null);
    if (e) {
        Object.keys(e).forEach(function (k) {
            if (k !== 'default') {
                var d = Object.getOwnPropertyDescriptor(e, k);
                Object.defineProperty(n, k, d.get ? d : {
                    enumerable: true,
                    get: function () { return e[k]; }
                });
            }
        });
    }
    n.default = e;
    return Object.freeze(n);
}

var RA__namespace = /*#__PURE__*/_interopNamespaceCompat(RA);
var Predicate__namespace = /*#__PURE__*/_interopNamespaceCompat(Predicate);

function expressionFactory(writer) {
    const curried = {
        object: curryWriter(objectWriter),
        objectProperty: curryWriter(createKeyValuePair),
        primitive: curryWriter(createPrimitive),
        identifier: curryWriter(createIdentifier),
        nextToken: curryWriter(createNextToken),
        array: curryWriter(arrayWriter),
        writer,
    };
    return curried;
    function curryWriter(cb) {
        return Function.pipe(cb, Function.apply(writer));
    }
    function createNextToken(writer) {
        return (x) => {
            if (isStringLiteral(x)) {
                return curried.identifier(x);
            }
            if (isPrimitive(x)) {
                return curried.primitive(x);
            }
            if (RA__namespace.isArray(x))
                return curried.array(x);
            if (Predicate__namespace.isRecord(x))
                return curried.object(x);
            if (Predicate__namespace.isNullable(x))
                return curried.identifier(`${x}`);
            return writer;
        };
    }
    function createIdentifier(writer) {
        return (x) => {
            return writer.write(`${x}`);
        };
    }
    function createPrimitive(writer) {
        return (primitive) => {
            if (typeof primitive === 'string') {
                return writer.quote(primitive);
            }
            return writer.write(`${primitive}`);
        };
    }
    function createKeyValuePair(writer) {
        return ([key, value]) => {
            return writer.hangingIndent(() => Function.pipe(writer.write(key).write(':').space(), Function.flip(createNextToken)(value)));
        };
    }
    function arrayWriter(writer) {
        return (arrayValue) => {
            return Function.pipe(writer.write('['), (w) => Function.pipe(arrayValue, RA__namespace.forEach((ww, i) => {
                curried.nextToken(ww).conditionalWrite(i + 1 < arrayValue.length, ',');
            }), () => w), (w) => w.write(']'));
        };
    }
    function objectWriter(writer) {
        return (object) => {
            const entries = Function.pipe(object, Object.entries, RA__namespace.filter((x) => !Predicate__namespace.isFunction(x[1]) && !Predicate__namespace.isPromise(x[1])));
            return writer.block(() => {
                Function.pipe(entries, RA__namespace.forEach((x, i) => {
                    Function.pipe(x, curried.objectProperty, (w) => w
                        .conditionalWrite(i + 1 < entries.length, ',')
                        .conditionalNewLine(i < entries.length));
                }));
            });
        };
    }
}
const isStringLiteralLike = (s) => s.startsWith('`');
const isStringLiteral = Function.pipe(Predicate__namespace.isString, Predicate__namespace.compose(isStringLiteralLike));
const isStringOrNumber = Predicate__namespace.or(Predicate__namespace.isString, Predicate__namespace.isNumber);
const isPrimitive = Predicate__namespace.or(isStringOrNumber, Predicate__namespace.isBoolean);

exports.expressionFactory = expressionFactory;
//# sourceMappingURL=writer.factory.js.map
