'use strict';

var upstreamTransformer = require('@expo/metro-config/babel-transformer');
var Effect = require('effect/Effect');
var Layer = require('effect/Layer');
var LogLevel = require('effect/LogLevel');
var Logger = require('effect/Logger');
var services = require('../services');
var jsx_visitors = require('./ast/jsx.visitors');

function _interopDefaultCompat (e) { return e && typeof e === 'object' && 'default' in e ? e : { default: e }; }

function _interopNamespaceCompat(e) {
    if (e && typeof e === 'object' && 'default' in e) return e;
    var n = Object.create(null);
    if (e) {
        Object.keys(e).forEach(function (k) {
            if (k !== 'default') {
                var d = Object.getOwnPropertyDescriptor(e, k);
                Object.defineProperty(n, k, d.get ? d : {
                    enumerable: true,
                    get: function () { return e[k]; }
                });
            }
        });
    }
    n.default = e;
    return Object.freeze(n);
}

var upstreamTransformer__default = /*#__PURE__*/_interopDefaultCompat(upstreamTransformer);
var Effect__namespace = /*#__PURE__*/_interopNamespaceCompat(Effect);
var Layer__namespace = /*#__PURE__*/_interopNamespaceCompat(Layer);
var LogLevel__namespace = /*#__PURE__*/_interopNamespaceCompat(LogLevel);
var Logger__namespace = /*#__PURE__*/_interopNamespaceCompat(Logger);

const mainProgram = Effect__namespace.gen(function* () {
    const ctx = yield* services.MetroCompilerContext;
    const transformer = yield* services.BabelTransformerService;
    yield* services.NativeTwinService;
    if (transformer.isNotAllowedPath(ctx.filename)) {
        return ctx.code;
    }
    const compiled = yield* jsx_visitors.transformJSXFile(ctx.code);
    return compiled.generated;
});
const MainLayer = services.BabelTransformerServiceLive.pipe(Layer__namespace.merge(Logger__namespace.replace(Logger__namespace.defaultLogger, services.BabelLogger)));
const babelRunnable = Effect__namespace.scoped(mainProgram.pipe(Logger__namespace.withMinimumLogLevel(LogLevel__namespace.All), Effect__namespace.provide(MainLayer)));
const transform = async (params) => {
    // console.log(inspect(params.options, false, null, true));
    return babelRunnable.pipe(Effect__namespace.provide(services.MetroCompilerContext.make(params, {
        componentID: true,
        styledProps: true,
        templateStyles: true,
        tree: true,
        order: true,
    })), Effect__namespace.provide(services.NativeTwinService.make(params.options)), Effect__namespace.map((code) => {
        // @ts-expect-error
        return upstreamTransformer__default.default.transform({
            src: code,
            options: params.options,
            filename: params.filename,
        });
    }), Effect__namespace.runPromise);
};

exports.babelRunnable = babelRunnable;
exports.transform = transform;
//# sourceMappingURL=babel.transformer.js.map
