'use strict';

var RA = require('effect/Array');
var Effect = require('effect/Effect');
var Function = require('effect/Function');
var Hash = require('effect/Hash');
var HashMap = require('effect/HashMap');
var Option = require('effect/Option');
var tree = require('@native-twin/helpers/tree');
var JSXImport_service = require('../jsx-import/JSXImport.service');
var models = require('../models');
var jsx_builder = require('./ast/jsx.builder');
var jsx_maps = require('./ast/jsx.maps');

function _interopNamespaceCompat(e) {
    if (e && typeof e === 'object' && 'default' in e) return e;
    var n = Object.create(null);
    if (e) {
        Object.keys(e).forEach(function (k) {
            if (k !== 'default') {
                var d = Object.getOwnPropertyDescriptor(e, k);
                Object.defineProperty(n, k, d.get ? d : {
                    enumerable: true,
                    get: function () { return e[k]; }
                });
            }
        });
    }
    n.default = e;
    return Object.freeze(n);
}

var RA__namespace = /*#__PURE__*/_interopNamespaceCompat(RA);
var Effect__namespace = /*#__PURE__*/_interopNamespaceCompat(Effect);
var Hash__namespace = /*#__PURE__*/_interopNamespaceCompat(Hash);
var HashMap__namespace = /*#__PURE__*/_interopNamespaceCompat(HashMap);
var Option__namespace = /*#__PURE__*/_interopNamespaceCompat(Option);

const program = Effect__namespace.scoped(Effect__namespace.gen(function* () {
    const ctx = yield* JSXImport_service.JSXImportPluginContext;
    return {
        name: '@native-twin/twin-babel-plugin-jsx',
        pre() {
            this.trees = [];
        },
        post() {
            for (const jsxElement of this.trees) {
                const fileTrees = HashMap__namespace.fromIterable(transformBabelTree(jsxElement));
                Function.pipe(fileTrees, HashMap__namespace.forEach((value) => {
                    const { leave, runtimeSheet } = jsx_maps.getJSXCompiledTreeRuntime(value, Function.pipe(value.parentID, Option__namespace.flatMap((x) => HashMap__namespace.get(fileTrees, x))));
                    jsx_builder.addTwinPropsToElement(leave, runtimeSheet, {
                        componentID: true,
                        order: true,
                        styledProps: true,
                        templateStyles: true,
                    });
                }));
            }
        },
        visitor: {
            Program: {
                exit(_, state) {
                    if (state.trees !== this.trees) {
                        state.trees = this.trees;
                    }
                },
            },
            JSXElement(path, state) {
                if (!ctx.isValidFile(state.filename))
                    return;
                const hash = Hash__namespace.string(state.filename ?? 'Unknown');
                const uid = path.scope.generateUid();
                const parentTree = new tree.Tree({
                    order: -1,
                    babelNode: path.node,
                    uid: `${hash}_${uid}`,
                    cssImports: [],
                    source: jsx_maps.getJSXElementSource(path),
                    parentID: null,
                });
                jsx_maps.gelBabelJSXElementChildLeaves(path, parentTree.root);
                this.trees.push(parentTree);
                path.skip();
            },
        },
    };
    function transformBabelTree(tree) {
        const fileSheet = RA__namespace.empty();
        tree.traverse((leave) => {
            const { value } = leave;
            const runtimeData = jsx_maps.extractMappedAttributes(leave.value.babelNode);
            const model = new models.JSXElementNode({
                leave,
                order: value.order,
                filename: `${leave.id}`,
                runtimeData,
                twin: {
                    config: ctx.twin.config,
                    preflight: {},
                    context: ctx.twCtx,
                    tw: ctx.twin,
                },
            });
            fileSheet.push([model.id, model]);
        }, 'breadthFirst');
        return fileSheet;
    }
}));
// const layer = Logger.replace(Logger.defaultLogger, BabelLogger);
function nativeTwinBabelPluginJSX(_, options, cwd) {
    return program.pipe(
    // Logger.withMinimumLogLevel(LogLevel.All),
    // Effect.provide(layer),
    Effect__namespace.provide(JSXImport_service.JSXImportPluginContext.make(options, cwd)), Effect__namespace.runSync);
}

module.exports = nativeTwinBabelPluginJSX;
//# sourceMappingURL=jsx.plugin.js.map
