'use strict';

var template = require('@babel/template');
var t = require('@babel/types');
var CodeBlockWriter = require('code-block-writer');
var writer_factory = require('../ast/writer.factory');

function _interopDefaultCompat (e) { return e && typeof e === 'object' && 'default' in e ? e : { default: e }; }

function _interopNamespaceCompat(e) {
  if (e && typeof e === 'object' && 'default' in e) return e;
  var n = Object.create(null);
  if (e) {
    Object.keys(e).forEach(function (k) {
      if (k !== 'default') {
        var d = Object.getOwnPropertyDescriptor(e, k);
        Object.defineProperty(n, k, d.get ? d : {
          enumerable: true,
          get: function () { return e[k]; }
        });
      }
    });
  }
  n.default = e;
  return Object.freeze(n);
}

var template__default = /*#__PURE__*/_interopDefaultCompat(template);
var t__namespace = /*#__PURE__*/_interopNamespaceCompat(t);
var CodeBlockWriter__default = /*#__PURE__*/_interopDefaultCompat(CodeBlockWriter);

const runtimeEntriesToAst = (entries) => {
    const ast = template__default.default.ast(entries);
    try {
        let value;
        if (Array.isArray(ast))
            return;
        if (t__namespace.isExpressionStatement(ast)) {
            value = ast.expression;
        }
        if (t__namespace.isBlockStatement(ast)) {
            const firstBlock = ast.body[0];
            if (t__namespace.isExpression(firstBlock)) {
                value = firstBlock;
            }
            if (t__namespace.isExpressionStatement(firstBlock)) {
                value = firstBlock.expression;
            }
        }
        if (!value) {
            return null;
        }
        return value;
    }
    catch {
        return null;
    }
};
/*
export const entriesToComponentData = (id: string, entries: RuntimeComponentEntry[]) => {
  const { writer, identifier, array } = expressionFactory(new CodeBlockWriter());
  const templateEntries = expressionFactory(new CodeBlockWriter());
  templateEntries.writer.block(() => {
    templateEntries.writer.write('[');
    entries
      .filter((x) => x.templateLiteral)
      .forEach((x) => {
        if (x.templateLiteral) {
          templateEntries.writer.block(() => {
            templateEntries.writer.writeLine(`id: "${id}",`);
            templateEntries.writer.writeLine(`target: "${x.target}",`);
            templateEntries.writer.writeLine(`prop: "${x.prop}",`);
            templateEntries.writer.writeLine(
              `entries: require('@native-twin/core').tw(${x.templateLiteral}),`,
            );
            templateEntries.writer.write(`templateLiteral: ${x.templateLiteral},`);
          });
        }
      });
    templateEntries.writer.write(']');
  });
  const styledProp = writer
    .block(() => {
      identifier(`require('@native-twin/jsx').StyleSheet.registerComponent("${id}", `);
      array(
        entries.map((x) => {
          return {
            templateLiteral: null,
            prop: x.prop,
            target: x.target,
            entries: x.entries,
            rawSheet: x.rawSheet,
          };
        }),
      );
      identifier(`)`);
    })
    .toString();
  return {
    styledProp,
    templateEntries: templateEntries.writer.toString(),
  };
};
*/
const entriesToComponentData = (id, entries) => {
    const { writer, identifier } = writer_factory.expressionFactory(new CodeBlockWriter__default.default());
    const styledProp = writer
        .block(() => {
        // identifier(`require('@native-twin/jsx').StyleSheet.registerComponent("${id}", `);
        identifier(`globalStyles?.["${id}"]`);
    })
        .toString();
    return {
        styledProp,
        templateEntries: null,
    };
};
const entriesToObject = (id, entries) => {
    const { writer, identifier, array } = writer_factory.expressionFactory(new CodeBlockWriter__default.default());
    const styledProp = writer
        .block(() => {
        // identifier(`require('@native-twin/jsx').StyleSheet.registerComponent("${id}", `);
        identifier(`StyleSheet.registerComponent("${id}", `);
        array(entries.map((x) => {
            return {
                templateLiteral: null,
                prop: x.prop,
                target: x.target,
                entries: x.entries,
                rawSheet: x.rawSheet,
            };
        }));
        identifier(`)`);
    })
        .toString();
    const templateEntries = entries.filter((x) => x.templateLiteral);
    // console.log('TEMPLATE_ENTRIES: ', templateEntries.length);
    if (templateEntries.length > 0) {
        const templateEntriesWriter = writer_factory.expressionFactory(new CodeBlockWriter__default.default());
        templateEntriesWriter.writer.block(() => {
            templateEntriesWriter.writer.write('[');
            templateEntries.forEach((x) => {
                if (x.templateLiteral) {
                    templateEntriesWriter.writer.block(() => {
                        templateEntriesWriter.writer.writeLine(`id: "${id}",`);
                        templateEntriesWriter.writer.writeLine(`target: "${x.target}",`);
                        templateEntriesWriter.writer.writeLine(`prop: "${x.prop}",`);
                        templateEntriesWriter.writer.writeLine(`entries: require('@native-twin/core').tw(${x.templateLiteral}),`);
                        templateEntriesWriter.writer.write(`templateLiteral: ${x.templateLiteral},`);
                    });
                }
            });
            templateEntriesWriter.writer.write(']');
        });
        return {
            styledProp,
            templateEntries: templateEntriesWriter.writer.toString(),
        };
    }
    return {
        styledProp,
        templateEntries: null,
    };
};

exports.entriesToComponentData = entriesToComponentData;
exports.entriesToObject = entriesToObject;
exports.runtimeEntriesToAst = runtimeEntriesToAst;
//# sourceMappingURL=twin.maps.js.map
