'use strict';

var Equal = require('effect/Equal');
var Function = require('effect/Function');
var Hash = require('effect/Hash');
var Option = require('effect/Option');
var jsx = require('@native-twin/css/jsx');
var jsx_maps = require('../jsx/ast/jsx.maps');
var twin = require('../jsx/twin');

function _interopNamespaceCompat(e) {
    if (e && typeof e === 'object' && 'default' in e) return e;
    var n = Object.create(null);
    if (e) {
        Object.keys(e).forEach(function (k) {
            if (k !== 'default') {
                var d = Object.getOwnPropertyDescriptor(e, k);
                Object.defineProperty(n, k, d.get ? d : {
                    enumerable: true,
                    get: function () { return e[k]; }
                });
            }
        });
    }
    n.default = e;
    return Object.freeze(n);
}

var Equal__namespace = /*#__PURE__*/_interopNamespaceCompat(Equal);
var Hash__namespace = /*#__PURE__*/_interopNamespaceCompat(Hash);
var Option__namespace = /*#__PURE__*/_interopNamespaceCompat(Option);

const jsxElementHash = (path, filename) => {
    const filenameHash = Hash__namespace.string(`${filename}`);
    return Function.pipe(Hash__namespace.string(`${path.start}-${path.end}`), Hash__namespace.combine(filenameHash), Hash__namespace.combine(Hash__namespace.string(path.type)));
};
class JSXElementNodeKey {
    path;
    filename;
    constructor(path, filename) {
        this.path = path;
        this.filename = filename;
    }
    [Hash__namespace.symbol]() {
        return jsxElementHash(this.path, this.filename);
    }
    [Equal__namespace.symbol](that) {
        return (that instanceof JSXElementNodeKey &&
            this.path.loc === that.path.loc &&
            this.path.type === that.path.type &&
            this.path.start == that.path.start);
    }
}
const jsxElementNodeKey = (path, filename) => new JSXElementNodeKey(path, filename);
class JSXElementNode {
    leave;
    order;
    filename;
    id;
    runtimeData;
    entries;
    childEntries;
    constructor({ filename, leave, order, runtimeData, twin: twin$1 }) {
        this.leave = leave;
        this.filename = filename;
        this.order = order;
        this.runtimeData = runtimeData;
        // this.id = `${jsxElementHash(leave.value.babelNode, filename)}`;
        this.id = leave.value.uid;
        this.entries = Function.pipe(twin.getElementEntries(this.runtimeData, twin$1.tw, twin$1.context));
        this.childEntries = jsx.getChildRuntimeEntries(this.entries);
    }
    get source() {
        return this.leave.value.source;
    }
    get parentSize() {
        return Function.pipe(this.parentLeave, Option__namespace.map((x) => x.childrenCount), Option__namespace.getOrElse(() => 0));
    }
    get parentLeave() {
        return Option__namespace.fromNullable(this.leave.parent);
    }
    get parentID() {
        return Option__namespace.fromNullable(this.leave.value.parentID);
    }
    get path() {
        return this.leave.value.babelNode;
    }
    get attributes() {
        return jsx_maps.getJSXElementAttrs(this.path);
    }
    get openingElement() {
        return this.path.openingElement;
    }
    get elementName() {
        return Function.pipe(jsx_maps.getJSXElementName(this.path.openingElement), Option__namespace.getOrElse(() => 'UnknownJSX'));
    }
    [Hash__namespace.symbol]() {
        return Hash__namespace.hash(this.id);
    }
    [Equal__namespace.symbol](that) {
        return that instanceof JSXElementNode && this.id === that.id;
    }
}

exports.JSXElementNode = JSXElementNode;
exports.JSXElementNodeKey = JSXElementNodeKey;
exports.jsxElementNodeKey = jsxElementNodeKey;
//# sourceMappingURL=JSXElement.model.js.map
