'use strict';

var upstreamTransformer = require('@expo/metro-config/babel-transformer');
var Context = require('effect/Context');
var Effect = require('effect/Effect');
var Function = require('effect/Function');
var HashMap = require('effect/HashMap');
var Layer = require('effect/Layer');
var Option = require('effect/Option');
var micromatch = require('micromatch');
var path = require('node:path');
var jsx_builder = require('../jsx/ast/jsx.builder');
var jsx_maps = require('../jsx/ast/jsx.maps');
var NativeTwin_service = require('./NativeTwin.service');

function _interopDefaultCompat (e) { return e && typeof e === 'object' && 'default' in e ? e : { default: e }; }

function _interopNamespaceCompat(e) {
    if (e && typeof e === 'object' && 'default' in e) return e;
    var n = Object.create(null);
    if (e) {
        Object.keys(e).forEach(function (k) {
            if (k !== 'default') {
                var d = Object.getOwnPropertyDescriptor(e, k);
                Object.defineProperty(n, k, d.get ? d : {
                    enumerable: true,
                    get: function () { return e[k]; }
                });
            }
        });
    }
    n.default = e;
    return Object.freeze(n);
}

var upstreamTransformer__default = /*#__PURE__*/_interopDefaultCompat(upstreamTransformer);
var Context__namespace = /*#__PURE__*/_interopNamespaceCompat(Context);
var Effect__namespace = /*#__PURE__*/_interopNamespaceCompat(Effect);
var HashMap__namespace = /*#__PURE__*/_interopNamespaceCompat(HashMap);
var Layer__namespace = /*#__PURE__*/_interopNamespaceCompat(Layer);
var Option__namespace = /*#__PURE__*/_interopNamespaceCompat(Option);
var micromatch__default = /*#__PURE__*/_interopDefaultCompat(micromatch);
var path__default = /*#__PURE__*/_interopDefaultCompat(path);

class MetroCompilerContext extends Context__namespace.Tag('metro/babel/transformer-context')() {
    static make = ({ filename, options, src }, generate) => Layer__namespace.effect(MetroCompilerContext, Effect__namespace.gen(function* () {
        const twin = yield* NativeTwin_service.NativeTwinService;
        const outputCSS = options.customTransformOptions.outputCSS;
        const platform = options.platform;
        return {
            generate,
            options,
            outputCSS,
            code: src,
            filename,
            inputCSS: options.customTransformOptions.inputCSS,
            allowedPaths: twin.config.content.map((x) => path__default.default.resolve(options.projectRoot, path__default.default.join(x))),
            platform,
        };
    }));
}
class BabelTransformerService extends Context__namespace.Tag('babel/TransformerService')() {
}
const BabelTransformerServiceLive = Layer__namespace.effect(BabelTransformerService, Effect__namespace.gen(function* () {
    const ctx = yield* MetroCompilerContext;
    return {
        // compileCode: (code) => twinBabelPluginTransform(code),
        isNotAllowedPath: (file) => {
            return !micromatch__default.default.isMatch(path__default.default.resolve(ctx.options.projectRoot, file), ctx.allowedPaths);
        },
        isCssFile: (filePath) => {
            return /\.css$/.test(filePath);
        },
        transformLeave: (trees) => {
            return HashMap__namespace.map(trees, (node) => {
                const { leave, runtimeSheet } = jsx_maps.getJSXCompiledTreeRuntime(node, Function.pipe(node.parentID, Option__namespace.flatMap((x) => HashMap__namespace.get(trees, x))));
                const runtimeAST = jsx_builder.addTwinPropsToElement(leave, runtimeSheet, ctx.generate);
                return { leave, runtimeSheet, runtimeAST };
            });
        },
        transform: (code) => {
            // @ts-expect-error
            return upstreamTransformer__default.default.transform({
                src: code,
                options: ctx.options,
                filename: ctx.filename,
            });
        },
    };
}));
// const twinBabelPluginTransform = (code: string) =>
//   Effect.gen(function* () {
//     const ctx = yield* BabelTransformerContext;
//     const generated = transformSync(code, {
//       // parserOpts: {
//       //   plugins: ['jsx', 'typescript'],
//       // },
//       code: false,
//       plugins: [
//         [
//           nativeTwinJSXBabelPlugin,
//           {
//             twinConfigPath: './tailwind.config.ts',
//           },
//         ],
//       ],
//       sourceType: 'unambiguous',
//       filename: ctx.filename,
//       ast: true,
//       cwd: ctx.options.projectRoot,
//       envName: process.env['NODE_ENV'],
//       minified: false,
//       generatorOpts: {
//         minified: false,
//       },
//       compact: false,
//     });
//     return Option.fromNullable(generated?.ast);
//     function nativeTwinJSXBabelPlugin(
//       _: BabelAPI,
//       options: TwinBabelOptions,
//       cwd: string,
//     ): PluginObj<TwinVisitorsState> {
//       return {
//         inherits: babelJSX,
//         pre() {
//           this.visited = HashMap.empty();
//           this.tree = {
//             _tag: 'tree',
//             value: [],
//           };
//         },
//         visitor: {
//           // Program: {
//           //   exit(state) {
//           //     this;
//           //     console.log('THESSS', state);
//           //   },
//           // },
//           JSXElement: {
//             exit() {
//               if (this.tree.value.length > 1) {
//                 const child = this.tree.value.pop();
//                 const parent = this.tree.value[this.tree.value.length - 1];
//                 if (child && parent) {
//                   parent.childs.push(child);
//                 }
//               }
//             },
//             enter(path) {
//               // console.log('STATE: ', state, this);
//               const key = babelJsxElementNodeKey(path.node, ctx.filename);
//               const elementNode = pipe(
//                 this.visited,
//                 HashMap.get(key),
//                 Option.match({
//                   onNone: () => {
//                     console.log('NOT_CACHED');
//                     return new BabelJSXElementNode(path.node, 0, ctx.filename, null);
//                   },
//                   onSome: (a) => {
//                     console.log('CACHED: ');
//                     return a;
//                   },
//                 }),
//               );
//               const sheet = elementNode.getTwinSheet(
//                 ctx.twin,
//                 ctx.twinCtx,
//                 HashSet.size(elementNode.childs),
//               );
//               addTwinPropsToElement(elementNode, sheet.propEntries, ctx.generate);
//               // updateVisitedNodes(key, elementNode, sheet);
//               this.visited = pipe(
//                 this.visited,
//                 HashMap.set(key, elementNode),
//                 HashMap.union(
//                   HashMap.fromIterable(
//                     HashSet.map(elementNode.childs, (x) =>
//                       Tuple.make(babelJsxElementNodeKey(x.path, ctx.filename), x),
//                     ),
//                   ),
//                 ),
//               );
//               this.tree.value.push({
//                 childs: [],
//                 filename: elementNode.filename,
//                 id: elementNode.id,
//                 node: getJSXElementName(elementNode.openingElement).pipe(
//                   Option.getOrElse(() => 'Unknown'),
//                 ),
//                 order: elementNode.order,
//                 parentNode: null,
//                 source: elementNode.binding(path).pipe(
//                   Option.map((x) => x.source),
//                   Option.getOrElse(() => 'Local'),
//                 ),
//               });
//             },
//           },
//         },
//       };
//     }
//   });

exports.BabelTransformerService = BabelTransformerService;
exports.BabelTransformerServiceLive = BabelTransformerServiceLive;
exports.MetroCompilerContext = MetroCompilerContext;
//# sourceMappingURL=MetroCompilerContext.service.js.map
